@@ -1,7 +1,12 @@
 from decimal import Decimal
 from datetime import date
 
-from django.test import TestCase
+from django.test import (
+        TestCase,
+        TransactionTestCase,
+        skipUnlessDBFeature,
+        )
+from django.utils.html import strip_tags
 
 import tablib
 
@@ -190,6 +195,22 @@ class ModelResourceTest(TestCase):
         full_title = resource.export_field(resource.get_fields()[0], self.book)
         self.assertEqual(full_title, '%s by %s' % (self.book.name, self.book.author.name))
 
+    def test_widget_fomat_in_fk_field(self):
+        class B(resources.ModelResource):
+
+            class Meta:
+                model = Book
+                fields = ('author__birthday',)
+                widgets = {
+                    'author__birthday': {'format': '%Y-%m-%d'},
+                }
+
+        author = Author.objects.create(name="Author")
+        self.book.author = author
+        resource = B()
+        result = resource.fields['author__birthday'].export(self.book)
+        self.assertEqual(result, str(date.today()))
+
     def test_widget_kwargs_for_field(self):
 
         class B(resources.ModelResource):
@@ -245,6 +266,38 @@ class ModelResourceTest(TestCase):
         self.assertIn(cat1, book.categories.all())
 
 
+class ModelResourceTransactionTest(TransactionTestCase):
+
+    def setUp(self):
+        self.resource = BookResource()
+
+    @skipUnlessDBFeature('supports_transactions')
+    def test_m2m_import_with_transactions(self):
+        cat1 = Category.objects.create(name='Cat 1')
+        headers = ['id', 'name', 'categories']
+        row = [None, 'FooBook', "%s" % cat1.pk]
+        dataset = tablib.Dataset(row, headers=headers)
+
+        result = self.resource.import_data(dataset, dry_run=True,
+                use_transactions=True)
+
+        row_diff = result.rows[0].diff
+        fields = self.resource.get_fields()
+
+        id_field = self.resource.fields['id']
+        id_diff = row_diff[fields.index(id_field)]
+        #id diff should exists because in rollbacked transaction
+        #FooBook has been saved
+        self.assertTrue(id_diff)
+
+        category_field = self.resource.fields['categories']
+        categories_diff = row_diff[fields.index(category_field)]
+        self.assertEqual(strip_tags(categories_diff), unicode(cat1.pk))
+
+        #check that it is really rollbacked
+        self.assertFalse(Book.objects.filter(name='FooBook'))
+
+
 class ModelResourceFactoryTest(TestCase):
 
     def test_create(self):
